@@ -1,10 +1,12 @@
 from dataclasses import dataclass, field
-from typing import List, Tuple, Hashable, Union, Iterable, Set
+from types import SimpleNamespace
+from typing import List, Tuple, Hashable, Union, Iterable, Set, Dict
 
 import numpy as np
 from ordered_set import OrderedSet
 
 from respice.analysis import UniqueEdgeMultiDiGraph
+from respice.collections import OneToManyInvertibleMapping
 from respice.components import Branch
 from respice.components.Branch import VoltageBranch, CurrentBranch
 
@@ -50,9 +52,13 @@ class MNAEquationStack:
     For more information about the MNA itself, see https://en.wikipedia.org/wiki/Modified_nodal_analysis
     """
 
-    def __init__(self, graph: UniqueEdgeMultiDiGraph, couplings: Iterable[List[Branch]]):
+    def __init__(self, graph: UniqueEdgeMultiDiGraph, couplings: Iterable[List[Branch]], references: Set[Hashable] = frozenset()):
+        self._cache = SimpleNamespace()
+        self._cache.equation_lambdified = None
+        self._cache.jacobian_lambdified = None
+
         self._graph: UniqueEdgeMultiDiGraph = graph
-        self._reference_nodes: Set[Hashable] = set()
+        self._reference_nodes = OneToManyInvertibleMapping()
 
         # Stores the vector-index relations to potential nodes.
         self._nodes: OrderedSet[Hashable] = OrderedSet()
@@ -68,12 +74,24 @@ class MNAEquationStack:
                                      for branch in coupled_branches}
 
         for subset in weakly_connected_components(graph):
-            # First element of a disjoint subset graph is always a reference potential.
+            # A bug in NetworkX prohibits editing the yielded sets directly, so we need to make a copy.
+            # See https://github.com/networkx/networkx/issues/4331.
+            subset = set(subset)
+
+            # Pin down ground node. Depending on the given preferred reference nodes, either the immediate next node is
+            # picked or one of the references specified. If more than one preferred ground node is specified that belong
+            # to the same subset, the first available one is picked.
+            preferred_ref_nodes = subset & references
+            if preferred_ref_nodes:
+                ref_node = preferred_ref_nodes.pop()
+                subset.remove(ref_node)
+            else:
+                ref_node = subset.pop()
+
+            for node in subset:
+                self._reference_nodes[ref_node] = node
 
             subset_iter = iter(subset)
-            # This one is safe and needs no StopIteration check. Otherwise there would be no subset yielded by
-            # weakly_connected_components.
-            self._reference_nodes.add(next(subset_iter))
 
             for potential in subset_iter:
                 self._nodes.add(potential)
@@ -243,6 +261,20 @@ class MNAEquationStack:
             (solution[self._nodes.index(source)] if source in self._nodes else 0)
         )
 
+    def get_reference_nodes(self):
+        """
+        Return all ground nodes (a.k.a. GND or reference nodes).
+        """
+        return self._reference_nodes.keys()
+
+    def get_node_voltage(self, solution: np.ndarray, node: Hashable):
+        return solution[self._nodes.index(node)]
+
+    def get_node_voltages(self, solution: np.ndarray):
+        vs = {node: self.get_node_voltage(solution, node) for node in self._nodes}
+        vs.update({gnd_node: 0 for gnd_node in self._reference_nodes})
+        return vs
+
     def get_voltage(self, solution: np.ndarray, branch: CurrentBranch):
         """
         Returns the voltage for a given current-branch from a calculated solution.
@@ -276,6 +308,56 @@ class MNAEquationStack:
         """
         return solution[self._currents.index(branch) + len(self._nodes)]
 
+    def assemble_vector(self, v: Dict[Hashable, float], i: Dict[VoltageBranch, float], fill: float = 0.0):
+        """
+        Assembles a proper vector from voltages and currents that can be used to evaluate this equation stack.
+
+        :param v:
+            Mapping of nodes to voltages.
+        :param i:
+            Mapping of voltage branches to their respective currents (other branch types are insignificant).
+        :param fill:
+            Fill value to use if a needed node or branch is missing.
+        :return:
+            A vector that can be used for `evaluate` and `jacobian` for equation evaluation.
+        """
+        vec = np.full(len(self), fill, dtype=float)
+        for node in v:
+            if node in self._nodes:
+                # Subtract also the reference potential if one was given explicitly to properly offset all voltages
+                # within a joint group of nodes.
+                vec[self._nodes.index(node)] = v[node] - v.get(self._reference_nodes.reverse[node], 0)
+
+        for branch in i:
+            if branch in self._currents:
+                vec[self._currents.index(branch) + len(self._nodes)] = i[branch]
+
+        return vec
+
+    def disassemble_vector(self, solution: np.ndarray):
+        """
+        Reverse operation of `assemble_vector`, creates `node -> potential` and `branch -> current` dictionaries
+        from a solution.
+
+        :param solution:
+            The solution to create the dictionaries from.
+        :return:
+            `node -> potential` and `branch -> current` dicts as a 2-tuple.
+        """
+        v = {}
+        i = {}
+
+        for branch in self._currents:
+            i[branch] = self.get_current(solution, branch)
+
+        for node in self._nodes:
+            v[node] = self.get_node_voltage(solution, node)
+
+        for gnd_node in self._reference_nodes:
+            v[gnd_node] = 0
+
+        return v, i
+
     def _create_lambdified_coupled_v_i_vector(self, coupled_branches: List[Union[VoltageBranch, CurrentBranch]]) -> List[str]:
         coupled_entries = []
 
@@ -310,6 +392,9 @@ class MNAEquationStack:
         :return:
             A function that is callable like `evaluate()` and behaves exactly like it.
         """
+        if self._cache.equation_lambdified is not None:
+            return self._cache.equation_lambdified
+
         mapped_branches = OrderedSet()
         code_globals = {'branches': mapped_branches}
 
@@ -359,7 +444,11 @@ class MNAEquationStack:
         # Compile the expression and get back the set up eq() function.
         code_object = compile('\n'.join(code), '<lambdified-mna-equation-stack>', 'exec')
         exec(code_object, code_globals)
-        return code_globals['eq']
+        fun = code_globals['eq']
+
+        self._cache.equation_lambdified = fun
+
+        return fun
 
     def jacobian(self, v_i: np.ndarray, t1: float, t2: float) -> np.ndarray:
         """
@@ -422,6 +511,9 @@ class MNAEquationStack:
         return np.array(jacobi_matrix)
 
     def lambdify_jacobian(self):
+        if self._cache.jacobian_lambdified is not None:
+            return self._cache.jacobian_lambdified
+
         mapped_branches = OrderedSet()
         code_globals = {'branches': mapped_branches}
 
@@ -502,4 +594,8 @@ class MNAEquationStack:
         # Compile the expression and get back the set up eq() function.
         code_object = compile('\n'.join(code), '<lambdified-mna-equation-stack-jacobian>', 'exec')
         exec(code_object, code_globals)
-        return code_globals['jac']
+        fun = code_globals['jac']
+
+        self._cache.jacobian_lambdified = fun
+
+        return fun
