@@ -161,3 +161,17 @@ class VoltageBranch(Branch):
             Present point in time (measured in seconds).
         """
         pass
+
+
+@dataclass(eq=False)
+class SwitchBranch(Branch):
+    def switch_state(self, t: float) -> bool:
+        """
+        The current switch state at time `t`.
+
+        :param t:
+            Absolute point in time (measured in seconds).
+        :return:
+            `True` if switch is on, `False` if off.
+        """
+        raise NotImplementedError
