@@ -1,4 +1,4 @@
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 
 from .TwoTerminalVoltageComponent import TwoTerminalVoltageComponent
 
@@ -22,18 +22,28 @@ class Capacitance(TwoTerminalVoltageComponent):
     """
     value: float
 
-    state_voltage: float = 0.0
-    state_current: float = 0.0
+    state_voltage: float = field(default=0.0, repr=False)
+    state_current: float = field(default=0.0, repr=False)
 
-    def get_voltage(self, i: float, dt: float) -> float:
+    def get_voltage(self, i: float, t1: float, t2: float) -> float:
+        dt = t2 - t1
         return self.state_voltage + (self.state_current + i) / (2 * self.value) * dt
 
-    def get_jacobian(self, i: float, dt: float) -> float:
+    def get_jacobian(self, i: float, t1: float, t2: float) -> float:
+        dt = t2 - t1
         return dt / (2 * self.value)
 
-    def update(self, i: float, dt: float):
-        self.state_voltage = self.get_voltage(i, dt)
+    def update(self, i: float, t1: float, t2: float):
+        self.state_voltage = self.get_voltage(i, t1, t2)
         self.state_current = i
 
+    @property
+    def state(self):
+        return self.state_voltage, self.state_current
+
+    @state.setter
+    def state(self, val):
+        self.state_voltage, self.state_current = val
+
 
 C = Capacitance
