@@ -1,7 +1,9 @@
 from __future__ import annotations
 
 from dataclasses import dataclass, field
-from typing import List, Hashable, Tuple
+from typing import List, Hashable, Tuple, Optional, Callable
+
+import numpy as np
 
 from .Branch import Branch
 
@@ -31,10 +33,10 @@ class Component:
         is "self-coupled").
 
         For example:
-        - A resistance is a self-coupled element. Its current is immediately depending on its voltage.
-        - A coupled inductance is a mutually-coupled element. Its current depends on the voltage of the other side
-          and vice versa. The edges describing the two coils on each side are mutually coupled by one
-          additional equation each.
+        * A resistance is a self-coupled element. Its current is immediately depending on its voltage.
+        * A coupled inductance is a mutually-coupled element. Its current depends on the voltage of the other side
+        and vice versa. The edges describing the two coils on each side are mutually coupled by one
+        additional equation each.
 
         :param graph:
             The graph to establish edges/connections in.
@@ -46,6 +48,39 @@ class Component:
         """
         raise NotImplementedError("I don't know how to connect this element between the given terminals")
 
+    def next_event(self, vi: Callable[[float], np.ndarray], t1: float, t2: float) -> Optional[float]:
+        r"""
+        Calculates a possible next event happening in between a calculation step.
+
+        An event is a significant point in time to consider to stop simulation at and perform an
+        additional step computation. Such significant events are for example discontinuities.
+
+        This function let's the user handle solving for events inside their respective components.
+        Due to different natures and behaviors of components, the solution can be better tied
+        individually (e.g. for continuous problems, how interpolation over step solution is performed
+        or immediate calculation for periodic sources).
+
+        :param vi:
+            The input voltages and currents as fed to the inner branches.
+
+            The parameter is a callable representing an interpolation function between `t1` and `t2`.
+            Since between `t1` and `t2` the exact solution is not known, the solver provides a suitable interpolation
+            function that can be used to calculate an event continuously, because often events also depend on the state
+            of a component.
+
+            It is not recommended to use values beyond the range :math:`[\texttt{t1}, \texttt{t2}]` because they are
+            extrapolated outside and may strongly deviate from the actual results.
+        :param t1:
+            Previous point in time (measured in seconds).
+        :param t2:
+            Present point in time (measured in seconds).
+        :return:
+            A time :math:`t` between `t1` and `t2` or `None` when no event would occur.
+
+            Broken events that lie outside the range :math:`[\texttt{t1}, \texttt{t2}]` will be ignored.
+        """
+        return None
+
     @property
     def state(self):
         """
