@@ -15,8 +15,8 @@ class CurrentSourceDC(TwoTerminalCurrentComponent):
     """
     value: float
 
-    def get_current(self, v: float, dt: float) -> float:
+    def get_current(self, v: float, t1: float, t2: float) -> float:
         return self.value
 
-    def get_jacobian(self, v: float, dt: float) -> float:
+    def get_jacobian(self, v: float, t1: float, t2: float) -> float:
         return 0.0
