@@ -1,5 +1,6 @@
 from dataclasses import dataclass
 from math import pi, sqrt
+from typing import Optional
 
 from .PeriodicCurrentSource import PeriodicCurrentSource
 
@@ -32,13 +33,14 @@ class CurrentSourceRectangular(PeriodicCurrentSource):
 
     @property
     def effective_amplitude(self):
-        """
+        r"""
         The effective amplitude (:math:`A_{RMS}`) depends not only on the amplitude, but also on the offset and duty
         cycle.
         According to the formula for RMS (see https://en.wikipedia.org/wiki/Root_mean_square and
         https://en.wikipedia.org/wiki/RMS_amplitude)
 
-        .. math:
+        .. math::
+
              A_{RMS} &= \sqrt{\frac{1}{T} \int_{0}^{T}{f^2(t) dt}} \\
                      &= \sqrt{
                             \frac{1}{T}
@@ -51,6 +53,7 @@ class CurrentSourceRectangular(PeriodicCurrentSource):
                             (o + A)^2 \delta T +
                             (o - A)^2 (1 - \delta) T
                         }
+
         :return:
             The effective amplitude current (measured in Amperes).
         """
@@ -59,6 +62,16 @@ class CurrentSourceRectangular(PeriodicCurrentSource):
             (self.offset - self.amplitude)**2 * (1 - self.duty)
         )
 
-    def get_signal(self, phase):
+    def get_signal(self, phase: float) -> float:
         high = phase < 2 * pi * self.duty
         return self.amplitude * (1 if high else -1) + self.offset
+
+    def next_signal_event(self, phase1: float, phase2: float) -> Optional[float]:
+        if phase1 == 0.0:
+            return 0.0
+
+        dutyphase = self.duty * 2 * pi
+        if phase1 <= dutyphase <= phase2:
+            return dutyphase
+
+        return None
