@@ -18,10 +18,10 @@ class TwoTerminalVoltageComponent(Component):
     you can immediately override methods like `get_voltage` or `update` and access voltages and currents via this
     component itself (`component.v` and `component.i`).
     """
-    _branch: TwoTerminalBranch = field(init=False, repr=False)
+    _branch: _TwoTerminalBranch = field(init=False, repr=False)
 
     @dataclass(eq=False)
-    class TwoTerminalBranch(VoltageBranch):
+    class _TwoTerminalBranch(VoltageBranch):
         """
         The branch that represents the two-terminal component.
 
@@ -41,7 +41,7 @@ class TwoTerminalVoltageComponent(Component):
             return f'{str(self.component)}'
 
     def __post_init__(self):
-        self._branch = self.TwoTerminalBranch(self)
+        self._branch = self._TwoTerminalBranch(self)
 
     def connect(self, terminal1: Hashable, terminal2: Hashable) -> List[Tuple[Hashable, Hashable, Branch]]:
         return [(terminal1, terminal2, self._branch)]
